#!BPY

"""
Name: 'Blender Rope Works SPINNER'
Blender: 236
Group: 'Animation'
Tooltip: 'Make ropes follow movements'
"""

#############################################################
#                                                           #
# Blender Rope Works                                        #
#                                                           #
# (C) February 2005 Stefano <S68> Selleri                   #
# Released under the Blender Artistic Licence (BAL)         #
# See www.blender.org                                       #
#                                                           #
# This script is to place ON FRAME CHANGE scene script link #
# It assumes the presence of N bezier curves named 'Rope.#' #
# being '#' an integer from 1 to N.                         #
#                                                           #
# For each Rope there are a number of empties. 'EndA.#' and #
# 'EndB.#' define starting and ending point of Bezier       #
# For each rope there are 1...M empties named 'P.#.#' the   #
# first '#' is an integer corresponding to the rope index,  #
# the second '#' corresponding to the 1...M integer index.  #
# Then we need as many empties 'C.#.#' these are mobile     #
# empties linked to the center of pulleys and are placed    #
# at a given distance from the corresponding 'P.#.#'        #
# Rope.# must have 2*M+2 control points.                    #
# Remember to CLEAR ROTATION on ropes!!!!                   #
#############################################################
# History                                                   #
# V: 0.0.0 - 24-02-05 - The script starts to take shape, an #
#                       history is now deserved :)          #
#            27-02-05 - All basic stuff now works           #
#            06-03-05 - Added more accurate lead-in lead-out#
#    0.0.1 - 09-03-05 - Major re-work for handling parents! #
#    0.0.2 - 14-03-05 - Now we use Registry to talk with GUI#
#############################################################

import Blender
from   Blender import *
from   math import *
import re

#############################################################
# GLOBALS                                                   #
#############################################################

#############################################################
# LINEAR ALGEBRA LIB                                        #
#############################################################

def vadd(A,B):
    ########## Vector ADD
    v = [0, 0, 0]
    
    v[0] = B[0]+A[0]
    v[1] = B[1]+A[1]
    v[2] = B[2]+A[2]

    return (v)

def vdif (A,B):
    ########## Vector DIFF
    v = [0, 0, 0]
    
    v[0] = B[0]-A[0]
    v[1] = B[1]-A[1]
    v[2] = B[2]-A[2]

    return (v)

def versor (v):
    ########## Vector -> Versor transformation
    vv = [0, 0, 0]
    n = sqrt(dotprod(v,v))

    if (n>0.000001):
        vv[0] = v[0] / n
        vv[1] = v[1] / n
        vv[2] = v[2] / n
    else:
        vv[0] = 0.
        vv[1] = 0.
        vv[2] = 0.

    return (vv)
    
def outerprod (a, v):
    ########## Vector x scalar -> Vector
    vv = [0, 0, 0]

    vv[0] = v[0] * a
    vv[1] = v[1] * a
    vv[2] = v[2] * a

    return (vv)

def dotprod(A,B):
    ########## Vector x Vector -> scalar
    v = A[0]*B[0] + A[1]*B[1] + A[2]*B[2]
    return (v)

def crossprod(A,B):
    ########## Vector x Vector -> Vector    
    v = [0, 0, 0]
    
    v[0] = A[1]*B[2] - A[2]*B[1]
    v[1] = A[2]*B[0] - A[0]*B[2]
    v[2] = A[0]*B[1] - A[1]*B[0]

    return (v)

#############################################################
# Deparents / Parents                                       #
#############################################################
def getGlobalPos (ob):

    g = [0,0,0] 
    m = ob.getMatrix()
    g[0] = m[3][0]
    g[1] = m[3][1]
    g[2] = m[3][2]

    return (g)

def setGlobalPos (ob,Gpos):

    try:
        P = ob.getParent()
	m = P.getInverseMatrix()

	ll = [0,0,0]

	ll[0] = Gpos[0]*m[0][0] + Gpos[1]*m[1][0] + Gpos[2]*m[2][0] + m[3][0]
	ll[1] = Gpos[0]*m[0][1] + Gpos[1]*m[1][1] + Gpos[2]*m[2][1] + m[3][1]
        ll[2] = Gpos[0]*m[0][2] + Gpos[1]*m[1][2] + Gpos[2]*m[2][2] + m[3][2]

    except:
        ll = Gpos

    ob.setLocation(ll)

    
#############################################################
# Computes positions for point C                            #
#############################################################
def Cplace(P0,P1,P2,l):
    
    v1 = versor(vdif(P0,P1))
    v2 = versor(vdif(P0,P2))
    n  = versor(crossprod(v1,v2))
    bi = versor(vadd(v1,v2))
    
    return(outerprod(l,bi),n)

#############################################################
# Computes tangents at pulley in point C                    #
#############################################################
def MakeReference (P,C,r,n,v):

    q = vdif(P,C)
    if (dotprod(q,q)>1e-6):
        q = versor(q)
        t = versor(crossprod(v,n))
        if (dotprod(t,q)>0):
            t = versor(crossprod(n,v))
    else:
        t = [0., 0., 0.]
        
    return (q,t)

def FindTangents (P1,C1,r1,n1,P2,C2,r2,n2):

    v  = versor(vdif(P1,P2))
    [q1,t1] = MakeReference(P1,C1,r1,n1,v)
    [q2,t2] = MakeReference(P2,C2,r2,n2,v)

    if (dotprod(q1,q1)<1e-6):
        #### E' un end, ovvero una carrucola di raggio 0
        T1 = P1
    else:
        #### E' una carrucola di raggio diverso da 0
        T1 = vadd(C1,outerprod(r1,t1))
                  
    if (dotprod(q2,q2)<1e-6):
        #### E' un end, ovvero una carrucola di raggio 0
        T2 = P2
    else:
        #### E' una carrucola di raggio diverso da 0
        T2 = vadd(C2,outerprod(r2,t2))

    return (T1,T2)

#############################################################
# MAIN                                                      #
#############################################################

fn    =  Get('curframe')

OL = Object.Get();

EndA = {}
EndB = {}
P    = {}
C    = {}

EndAgl = {}
EndBgl = {}
Pgl    = {}
Cgl    = {}

nn    = {}
Rope = {}

Ropegl = {}

##### PERSISTENT DATA
dict = Registry.GetKey('BRW')
try:
    dis = dict['dis']
except:
    dis = {}
    
try:
    rad = dict['rad']
except:
    rad = {}

#dis = [[],[0,1., 1.5]]
#rad = [[],[0,0.5, 1.]]

#dis = [[],[0,0.14,0.14,0.14,0.14],[0,0.14,0.14,0.14,0.14],[0,0.14,0.14]]
#rad = [[],[0,0.065,0.065,0.065,0.065],[0,0.065,0.065,0.065,0.065],[0,0.065,0.065]]

########## Extracts Ends, Points, Centers, Ropes
for o in OL:
    try:
        i = int(re.search(r"^EndA\.(\d*)",o.name).group(1))
        EndA[i]   = o
        EndAgl[i] = getGlobalPos(o)
    except:
        pass

    try:
        i = int(re.search(r"^EndB\.(\d*)",o.name).group(1))
        EndB[i] = o
        EndBgl[i] = getGlobalPos(o)
    except:
        pass

    try:
        i = int(re.search(r"^P\.(\d*)\.(\d*)",o.name).group(1))
        j = int(re.search(r"^P\.(\d*)\.(\d*)",o.name).group(2))
        try:
            P[i][j] = o
            Pgl[i][j] = getGlobalPos(o)
        except:
            P[i] = {}
            P[i][j] = o
            Pgl[i] = {}
            Pgl[i][j] = getGlobalPos(o)
    except:
        pass

    try:
        i = int(re.search(r"^C\.(\d*)\.(\d*)",o.name).group(1))
        j = int(re.search(r"^C\.(\d*)\.(\d*)",o.name).group(2))
        try:
            C[i][j] = o
            Cgl[i][j] = getGlobalPos(o)
        except:
            C[i] = {}
            C[i][j] = o
            Cgl[i] = {}
            Cgl[i][j] = getGlobalPos(o)
    except:
        pass

    try:
        i = int(re.search(r"^Rope\.(\d*)",o.name).group(1))
        Rope[i] = o
        Ropegl[i] = getGlobalPos(o)
    except:
        pass


########## Handling Ropes
for i in Rope.keys():
    print "+-> Handling Rope #",i

    ########## Placing C.i.j
    lj = max(P[i].keys())
    for j in P[i].keys():
        print "| +-> Handling Pulley center #",j

        if (lj==1):
            P1gl = EndAgl[i]
            P2gl = EndBgl[i]
        else:
            if (j==1):
                P1gl = EndAgl[i]
                P2gl = Pgl[i][j+1]
            elif (j==lj):
                P1gl = Pgl[i][j-1]
                P2gl = EndBgl[i]
            else:
                P1gl = Pgl[i][j-1]
                P2gl = Pgl[i][j+1]

        [Ca,na] = Cplace (Pgl[i][j],P1gl,P2gl,dis[i][j].val)

        setGlobalPos(C[i][j],vadd(Ca,Pgl[i][j]))
        try:
            nn[i][j] = na
        except:
            nn[i] = {}
            nn[i][j] = na
        
    ########## First Approximation of Tangent Points
    BC = Rope[i].getData()
    CN = Ropegl[i]

    print "| +-> Handling Bezier Stage 0"
    P1 = EndAgl[i]
    P2 = Pgl[i][1]
    C1 = EndAgl[i]
    C2 = getGlobalPos(C[i][1])
    r1 = 0
    r2 = rad[i][1].val
    n1 = [0., 0., 0.]
    n2 = nn[i][1]

    T = {}
    [T1,T2] = FindTangents(P1,C1,r1,n1,P2,C2,r2,n2)
    T[0] = {}
    T[0][0] = T1
    T[0][1] = T2

    for j in C[i].keys():

        if (j==lj):
            P1 = Pgl[i][j]
            P2 = EndBgl[i]
            C1 = getGlobalPos(C[i][j])
            C2 = EndBgl[i]
            r1 = rad[i][j].val
            r2 = 0
            n1 = nn[i][j]
            n2 = [0., 0., 0.]
        else:
            P1 = Pgl[i][j]
            P2 = Pgl[i][j+1]
            C1 = getGlobalPos(C[i][j])
            C2 = getGlobalPos(C[i][j+1])
            r1 = rad[i][j].val
            r2 = rad[i][j+1].val
            n1 = nn[i][j]
            n2 = nn[i][j+1]

        [T1,T2] = FindTangents(P1,C1,r1,n1,P2,C2,r2,n2)
        T[j] = {}
        T[j][0] = T1 
        T[j][1] = T2 

    ########## Refining Tangent Points
    print "| +-> Handling Bezier Stage 1"
    for j in [0]+C[i].keys():
        q = versor(vdif(T[j][0],T[j][1]))
        if (j==0):
            C0 = EndAgl[i]
            C1 = getGlobalPos(C[i][1])
        elif (j==lj):
            C0 = getGlobalPos(C[i][j])
            C1 = EndBgl[i]
        else:
            C0 = getGlobalPos(C[i][j])
            C1 = getGlobalPos(C[i][j+1])
        v = versor(vdif(C0,C1))
        nq = crossprod(q,v)
        theta = asin(sqrt(dotprod(nq,nq)))

        t0 = versor(vdif(C0,T[j][0])) 
        t1 = versor(vdif(C1,T[j][1])) 

        nt0 = versor(vadd(outerprod(cos(theta),t0),
                          outerprod(sin(theta),v)))
        nt1 = versor(vadd(outerprod(cos(theta),t1),
                          outerprod(-sin(theta),v)))

        try:
            r = rad[i][j].val
        except:
            r = 0.0
        T[j][0] = vadd(outerprod(r,nt0),C0)
        
        try:
            r = rad[i][j+1].val
        except:
            r = 0.0
        T[j][1] = vadd(outerprod(r,nt1),C1)

    ########## Setting Bezier Control Points
    BC.setFlag(1)

    print "| +-> Handling Bezier Stage 2"
    ## Fixed End 1
    T0 = vdif(CN,T[0][0])
    BC.setControlPoint(0,0,T0+T0+T0)
    ccp = 1
    for j in C[i].keys():
        ## First Point on Cj (Tangent)
        T1  = vdif(CN,T[j-1][1])
        Tm1 = vdif(CN,T[j-1][0])
        v1 = versor(vdif(Tm1,T1))

        ## Second Point on Cj (Middle)
        C0 = getGlobalPos(C[i][j])
        ta = versor(vdif(C0,T[j-1][1]))
        tb = versor(vdif(C0,T[j][0]))
        ti = versor(vadd(ta,tb))
        if (dotprod(ti,ti) < 0.1):
            print "Siamo nei guai..."
            Ta = C0
            theta = math.pi/2;
        else:
            if(dotprod(ti,vdif(C0,Pgl[i][j]))<0):
               ti = outerprod(-1,ti)
            n  = crossprod(ta,ti)
            bt = versor(crossprod(n,ti))
            theta = asin(dotprod(n,n))
            r = rad[i][j].val
            Ta  = vdif(CN,vadd(outerprod(r,ti),C0))

        ## Third Point on Cj (Tangent)
        T0 = vdif(CN,T[j][0])
        Tp1 = vdif(CN,T[j][1])
        v0 = versor(vdif(T0,Tp1))

        dd = (4.0/3.0)*sqrt((1-cos(theta/2.0))/(1+cos(theta/2.0)))
        T1a = vdif(outerprod(r*dd,v1),T1)
        T1b = vadd(outerprod(r*dd,v1),T1)
        BC.setControlPoint(0,ccp,T1a+T1+T1b)
        ccp = ccp + 1

        Taa = vdif(outerprod(r*dd,bt),Ta)
        Tab = vadd(outerprod(r*dd,bt),Ta)
        BC.setControlPoint(0,ccp,Taa+Ta+Tab)
        ccp = ccp + 1          

        T0a = vdif(outerprod(r*dd,v0),T0)
        T0b = vadd(outerprod(r*dd,v0),T0)
        BC.setControlPoint(0,ccp,T0a+T0+T0b)
        ccp = ccp + 1

    ## Fixed End 2
    T1 = vdif(CN,T[j][1])    
    BC.setControlPoint(0,ccp,T1+T1+T1)

    BC.update()
    #print T
    
Blender.Redraw()


#    T1 = vdif(CN,T1)
#    T2 = vdif(CN,T2)
#    v = versor(vdif(T2,T1))
#    BC.setControlPoint(0,1,T1+T1+T1)
#    T2a = vadd(T2,outerprod(r2*0.522,v))
#    T2b = vadd(T2,outerprod(-r2*0.522,v))
#    BC.setControlPoint(0,1,T2a+T2+T2b)
